using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.MvvmTips.ViewModels;
using Benday.MvvmTips.Models;
using Benday.MvvmTips.WcfDataAccess;

namespace Benday.MvvmTips.UnitTests
{
    [TestClass]
    public class PersonViewModelFixture : SilverlightUnitTestBase
    {
        protected override void OnTestInitialize()
        {
            m_AdapterInstance = null;
            m_ViewModelInstance = null;
            m_ModelInstance = null;
            m_RepositoryInstance = null;
        }

        [TestMethod]
        public void LoadByIdLoadsTheModelFromRepositoryAndPopulatesTheUi()
        {
            UnitTestUtility.PopulateWithTestData(ModelInstance);

            // set the value that the mock repository will return when
            // LoadById() is called
            RepositoryInstance.Reset(ModelInstance);

            int expectedPersonId = ModelInstance.Id;

            Assert.AreNotEqual<int>(0, expectedPersonId, 
                "ModelInstance Id should not be 0 at the start of the test.");

            Assert.IsFalse(RepositoryInstance.WasLoadByIdCalled, 
                "Should not have any calls to LoadById() yet.");


            // load the viewmodel by id
            ViewModelInstance.IdToLoad.Value = expectedPersonId;
            ViewModelInstance.LoadCommand.Execute(null);


            // did the viewmodel call the repository properly?
            Assert.IsTrue(RepositoryInstance.WasLoadByIdCalled, 
                "LoadById() on the repository wasn't called.");
            Assert.AreEqual<int>(expectedPersonId, 
                RepositoryInstance.IdReceivedByLoadByIdMethod, 
                "ViewModel didn't pass the expected Id value to the repository.");

            // did the viewmodel populate itself properly?
            UnitTestUtility.AssertAreEqual(ModelInstance, ViewModelInstance);
        }

        [TestMethod]
        public void CancelRevertsChangesMadeToTheViewModelValues()
        {
            UnitTestUtility.PopulateWithTestData(ModelInstance);
            RepositoryInstance.Reset(ModelInstance);
            ViewModelInstance.IdToLoad.Value = ModelInstance.Id;
            ViewModelInstance.LoadCommand.Execute(null);

            // The Model and ViewModel should be in sync
            UnitTestUtility.AssertAreEqual(ModelInstance, ViewModelInstance);


            // change the values on the ViewModel
            UnitTestUtility.PopulateWithTestData(ViewModelInstance);
            
            // The Model and ViewModel should NOT be in sync
            UnitTestUtility.AssertAreNotEqual(ModelInstance, ViewModelInstance);
            
            // Call the Cancel command 
            ViewModelInstance.CancelCommand.Execute(null);

            // did the ViewModel revert to the original values?
            UnitTestUtility.AssertAreEqual(ModelInstance, ViewModelInstance);
        }

        [TestMethod]
        public void SaveCommandCallsSaveOnRepositoryWithCorrectlyPopulatedModel()
        {
            RepositoryInstance.Reset();
            
            // set all the model-related values on the ViewModel
            UnitTestUtility.PopulateWithTestData(ViewModelInstance);
            
            // set the Id to zero to emulate a "create new person" scenario
            ViewModelInstance.Id.Value = 0;

            // Call the Save command 
            ViewModelInstance.SaveCommand.Execute(null);

            // did it work?
            Assert.AreNotEqual<int>(0, ViewModelInstance.Id.Value, "Id shouldn't be zero after save.");
            Assert.IsTrue(RepositoryInstance.WasSaveCalled, "Save wasn't called.");
            Assert.IsNotNull(RepositoryInstance.ModelReceivedBySaveMethod, "Repository got a null Model.");
            UnitTestUtility.AssertAreEqual(ViewModelInstance, RepositoryInstance.ModelReceivedBySaveMethod);
        }

        private PersonRepositoryMock m_RepositoryInstance;
        public PersonRepositoryMock RepositoryInstance
        {
            get
            {
                if (m_RepositoryInstance == null)
                {
                    // use an in-memory mock version of IPersonRepository 
                    // rather than a real implementation.  Why?
                    // 1) Because we're testing the implementation of the 
                    //    PersonViewModel and not the Wcf Repository or Service
                    // 2) No dependency on the WCF Service means less stuff to 
                    //    setup or break
                    // 3) Much easier to create the scenarios we want to test
                    //    if we can just create fake data
                    // 4) No cleanup of data from previous tests
                    // 5) Test the behavior of the viewmodel to verify that it
                    //    calls the Repository when we expect

                    m_RepositoryInstance = new PersonRepositoryMock();
                }
                return m_RepositoryInstance;
            }
        }

        private IPersonViewModel m_ViewModelInstance;
        public IPersonViewModel ViewModelInstance
        {
            get
            {
                if (m_ViewModelInstance == null)
                {
                    // "inject" our mock repository in to the viewmodel 
                    // constructor along with an instance of the adapter 
                    m_ViewModelInstance =
                        new PersonViewModel(
                            RepositoryInstance,
                            AdapterInstance);
                }

                return m_ViewModelInstance;
            }
        }

        private IPersonModelToViewModelAdapter m_AdapterInstance;
        public IPersonModelToViewModelAdapter AdapterInstance
        {
            get
            {
                if (m_AdapterInstance == null)
                {
                    m_AdapterInstance = new PersonModelToViewModelAdapter();
                }

                return m_AdapterInstance;
            }
        }

        private IPerson m_ModelInstance;
        public IPerson ModelInstance
        {
            get
            {
                if (m_ModelInstance == null)
                {
                    m_ModelInstance = new Person();
                }

                return m_ModelInstance;
            }
        }
    }
}
