﻿using System;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Benday.MvvmTips.WcfDataAccess;
using Benday.MvvmTips.Common;
using Benday.MvvmTips.Models;

namespace Benday.MvvmTips.ViewModels
{
    public class PersonViewModel : ViewModelBase, IPersonViewModel
    {
        public PersonViewModel() : 
            this(new AsyncWcfPersonRepository(), 
            new PersonModelToViewModelAdapter())
        {
            
        }

        public PersonViewModel(
            IPersonRepository repository, 
            IPersonModelToViewModelAdapter modelToViewModelAdapter) 
        {
            if (repository == null)
            {
                throw new ArgumentNullException(
                    "repository", "Argument cannot be null.");
            }

            if (modelToViewModelAdapter == null)
            {
                throw new ArgumentNullException(
                    "modelToViewModelAdapter", "Argument cannot be null.");
            }

            m_Repository = repository;
            m_ModelToViewModelAdapter = modelToViewModelAdapter;

            InitializeFields();
            InitializeCommands();

            ClearMessage();
        }

        private void InitializeFields()
        {
            IdToLoad = new ViewModelField<int>();
            Message = new ViewModelField<string>();
            Id = new ViewModelField<int>();
            FirstName = new ViewModelField<string>();
            LastName = new ViewModelField<string>();
            EmailAddress = new ViewModelField<string>();
        }

        private void InitializeCommands()
        {
            LoadCommand = new RelayCommand(Load);
            SaveCommand = new RelayCommand(Save);
            CancelCommand = new RelayCommand(Cancel);
            ValidateCommand = new RelayCommand(Validate);
        }

        private IPersonRepository m_Repository;
        private IPersonModelToViewModelAdapter m_ModelToViewModelAdapter;
        
        public ICommand LoadCommand { get; private set; }
        public ICommand SaveCommand { get; private set; }
        public ICommand CancelCommand { get; private set; }
        public ICommand ValidateCommand { get; private set; }

        private const string IdToLoadPropertyName = "IdToLoad";

        private ViewModelField<int> m_IdToLoad;
        public ViewModelField<int> IdToLoad
        {
            get
            {
                return m_IdToLoad;
            }
            set
            {
                m_IdToLoad = value;
                RaisePropertyChanged(IdToLoadPropertyName);
            }
        }

        private const string MessagePropertyName = "Message";

        private ViewModelField<string> m_Message;
        public ViewModelField<string> Message
        {
            get
            {
                return m_Message;
            }
            set
            {
                m_Message = value;
                RaisePropertyChanged(MessagePropertyName);
            }
        }

        private const string IdPropertyName = "Id";

        private ViewModelField<int> m_Id;
        public ViewModelField<int> Id
        {
            get
            {
                return m_Id;
            }
            set
            {
                m_Id = value;
                RaisePropertyChanged(IdPropertyName);
            }
        }

        private const string FirstNamePropertyName = "FirstName";

        private ViewModelField<string> m_FirstName;
        public ViewModelField<string> FirstName
        {
            get
            {
                return m_FirstName;
            }
            set
            {
                m_FirstName = value;
                RaisePropertyChanged(FirstNamePropertyName);
            }
        }

        private const string LastNamePropertyName = "LastName";

        private ViewModelField<string> m_LastName;
        public ViewModelField<string> LastName
        {
            get
            {
                return m_LastName;
            }
            set
            {
                m_LastName = value;
                RaisePropertyChanged(LastNamePropertyName);
            }
        }

        private const string EmailAddressPropertyName = "EmailAddress";

        private ViewModelField<string> m_EmailAddress;
        public ViewModelField<string> EmailAddress
        {
            get
            {
                return m_EmailAddress;
            }
            set
            {
                m_EmailAddress = value;
                RaisePropertyChanged(EmailAddressPropertyName);
            }
        }

        private void ShowMessage(Exception ex)
        {
            ShowMessage(ex.Message);
        }

        private void ShowMessage(string message)
        {
            Message.Value = message;
            Message.IsVisible = true;
        }

        private void ClearMessage()
        {
            Message.Value = String.Empty;
            Message.IsVisible = false;
        }

        private void Load()
        {
            try
            {
                ShowMessage("Loading...");

                m_Model = null;

                m_Repository.LoadById(new ReturnResult<IPerson>(LoadCompleted), IdToLoad.Value);
            }
            catch (Exception ex)
            {
                ShowMessage(ex);
            }
        }

        private IPerson m_Model = null;

        private void LoadCompleted(ReturnResult<IPerson> callback)
        {
            if (callback.Error != null)
            {
                ShowMessage(callback.Error);
            }
            else
            {
                var adapter = new PersonModelToViewModelAdapter();

                adapter.Adapt(callback.Result, this);

                m_Model = callback.Result;

                ClearMessage();
            }
        }

        private void Save()
        {
            try
            {
                ShowMessage("Saving...");

                if (m_Model == null)
                {
                    m_Model = new Person();
                }

                m_ModelToViewModelAdapter.Adapt(this, m_Model);

                m_Repository.Save(new ReturnResult<IPerson>(SaveCompleted), m_Model);    
            }
            catch (Exception ex)
            {
                ShowMessage(ex);
            }            
        }

        private void SaveCompleted(ReturnResult<IPerson> callback)
        {
            if (callback.Error != null)
            {
                ShowMessage(callback.Error);
            }
            else
            {
                m_ModelToViewModelAdapter.Adapt(callback.Result, this);

                m_Model = callback.Result;

                ClearMessage();
            }
        }

        private void Cancel()
        {
            if (m_Model != null)
            {
                // revert back to model and discard changes
                m_ModelToViewModelAdapter.Adapt(m_Model, this);    
            }            
        }

        private void Validate()
        {
            ToggleValidity(Id, "Id");
            ToggleValidity(FirstName, "FirstName");
            ToggleValidity(LastName, "LastName");
            ToggleValidity(EmailAddress, "EmailAddress");
        }

        private void ToggleValidity<T>(ViewModelField<T> field, string name)
        {
            if (field.IsValid == true)
            {
                field.IsValid = false;
                field.ValidationMessage = String.Format("'{0}' field contains an invalid value.", name);
            }
            else
            {
                field.IsValid = true;
            }
        }
    }
}